# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

import csv
import logging
import functools
from collections import defaultdict
from argparse import ArgumentParser, Namespace

# -------------------
# Third party imports
# -------------------º

import decouple

from lica.cli import execute
from lica.jinja2 import render_from
from lica.sqlalchemy import sqa_logging

from tessdbdao import ObserverType
from tessdbapi.model import ObserverInfo
from tessdbapi.noasync.observer import observer_create, observer_update


# --------------
# local imports
# -------------

from .._version import __version__
from . import parser as prs


from .dao import engine, Session

from ..admin import TessDbServer

# ---------
# Constants
# ---------

CREATE_TEMPLATE = "observer_create.sh.j2"

# ----------------
# Global variables
# ----------------

log = logging.getLogger(__name__.split(".")[-1])
package = ".".join(__name__.split(".")[:-2])

host = decouple.config("TESSDB_ADMIN_HOST", default="localhost")
port = decouple.config("TESSDB_ADMIN_PORT", cast=int, default=8080)

render = functools.partial(render_from, package)

# ================
# MAIN ENTRY POINT
# ================


def cli_observer_create(args: Namespace) -> None:
    candidate = ObserverInfo(
        type=args.type,
        name=args.name,
        affiliation=args.affiliation,
        acronym=args.acronym,
        website_url=args.website_url,
        email=args.email,
    )
    with TessDbServer(host=host, port=port, test=args.test):
        with Session() as session:
            with session.begin():
                log.info("Registering observer: %s", dict(candidate))
                observer_create(
                    session,
                    candidate,
                    args.dry_run,
                )


def cli_observer_update(args: Namespace) -> None:
    candidate = ObserverInfo(
        type=args.type,
        name=args.name,
        affiliation=args.affiliation,
        acronym=args.acronym,
        website_url=args.website_url,
        email=args.email,
    )
    with TessDbServer(host=host, port=port, test=args.test):
        with Session() as session:
            with session.begin():
                log.info("Updating observer: %s", dict(candidate))
                observer_update(
                    session,
                    candidate,
                    args.fix,
                    args.dry_run,
                )

def cli_observer_generate(args: Namespace) -> None:
    with open(args.input_csv, newline="") as csvfile:
        reader = csv.DictReader(csvfile)
        observers = [dict(row) for row in reader]
    try:
        observers = list(filter(lambda x: ObserverType(x["type"].title()) is not None, observers))
    except Exception:
        log.error("Observer Type is not valid")
    else:
        log.info("found %d records", len(observers))
        obs_by_name = defaultdict(list)
        for obs in observers:
            obs_by_name[obs["name"]].append(obs)
        observers = [val[0] for key, val in obs_by_name.items()]
        log.info("reduced to %d observers", len(observers))
        context = {"observers": observers}
        script = render(CREATE_TEMPLATE, context)
        with open(args.output_script, "w") as outfile:
            outfile.write(script)
        log.info("script '%s' generated", args.output_script)


def add_args(parser: ArgumentParser) -> None:
    subparser = parser.add_subparsers(dest="command", required=True)
    p = subparser.add_parser(
        "create",
        parents=[prs.observer(), prs.dry(), prs.test()],
        help="Create new observer",
    )
    p.set_defaults(func=cli_observer_create)
    p = subparser.add_parser(
        "update",
        parents=[prs.observer(), prs.dry(), prs.fix(), prs.test()],
        help="Update existing observer",
    )
    p.set_defaults(func=cli_observer_update)
    p = subparser.add_parser(
        "generate",
        parents=[
            prs.ifile(),
            prs.ofile(),
            prs.dry(),
        ],
        help="Generate observers creation script from observers CSV",
    )
    p.set_defaults(func=cli_observer_generate)


def cli_main(args: Namespace) -> None:
    sqa_logging(args)
    args.func(args)
    engine.dispose()


def main():
    execute(
        main_func=cli_main,
        add_args_func=add_args,
        name=__name__,
        version=__version__,
        description="TESSDB Observer tool",
    )



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

import logging
from datetime import datetime, timedelta

# -------------------
# Third party imports
# -------------------

import requests

# ----------------
# Global variables
# ----------------

log = logging.getLogger(__name__.split(".")[-2])


class TooCloseError(RuntimeError):
    """Time too close to tessdb database backup"""
    def __str__(self):
        s = self.__doc__
        if self.args:
            s = '{0}: {1}'.format(s, self.args[0])
        s = '{0}.'.format(s)
        return s

# Contexct manager tp coordinate access to the underlying database
# with TessDB server, making it to stop
# We do not cacth any exception, since we are not handling them and we need to abort anyway
# The timestamp constructor is only needed for testing purposes


class TessDbServer:
    BEFORE = timedelta(hours=1)
    AFTER = timedelta(minutes=30)

    def __init__(
        self, host: str, port: int, timeout=1, test: bool = False, timestamp: datetime = None
    ):
        self._host = host
        self._port = port
        self._pause_url = f"http://{host}:{port}/v1/server/pause"
        self._resume_url = f"http://{host}:{port}/v1/server/resume"
        self._timeout = timeout
        self._test = test
        self._tstamp = timestamp

    def _assert_timestamp(self) -> None:
        self._tstamp = datetime.now() if self._tstamp is None else self._tstamp
        noon = datetime.now().replace(hour=12, minute=0, second=0, microsecond=0)
        if noon - TessDbServer.BEFORE <= self._tstamp <= noon + TessDbServer.AFTER:
            raise TooCloseError(self._tstamp.strftime("%Y-%m-%d %H:%M:%S"))

    def __enter__(self) -> None:
        if not self._test:
            self._assert_timestamp()
            log.info("requesting a pause to tessdb server on %s,%d", self._host, self._port)
            r = requests.post(self._pause_url, data={}, timeout=self._timeout)
            r.raise_for_status()
            response = r.json()
            log.info(response)

    def __exit__(self, exc_type, exc_val, exc_tb) -> bool:
        if exc_type is None and not self._test:
            log.info("requesting tessdb server on %s:%d to resume", self._host, self._port)
            r = requests.post(self._resume_url, data={}, timeout=self._timeout)
            r.raise_for_status()
            response = r.json()
            log.info(response)
        elif exc_type is not None:
            log.error(
                "an exception has been registered withn the %s context manager",
                self.__class__.__name__,
            )
        return False



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

import os
import csv
import logging

from argparse import ArgumentParser, Namespace

# -------------------
# Third party imports
# -------------------

from lica.cli import execute
from lica.sqlalchemy import sqa_logging

# --------------
# local imports
# -------------

from .._version import __version__
from . import parser as prs
from .dao import engine_tdb, engine_zpt
from .logfile import collect_readings_for, collect_references

# ----------------
# Global variables
# ----------------

package = ".".join(__name__.split(".")[:-1])
log = logging.getLogger(__name__.split(".")[-1])

# ================
# MAIN ENTRY POINT
# ================


def cli_readings(args: Namespace) -> None:
    readings = collect_readings_for(args.name, args.log_dir, args.start, args.end, args.batch_size)
    if readings:
        log.info("start = %s, end = %s", readings[0]["tstamp"], readings[-1]["tstamp"])
        register_tstamp = readings[0]["tstamp"].replace(hour=0, minute=0, second=0, microsecond=0)
        log.info("Should register photometer at %s", register_tstamp)
        start = readings[0]["tstamp"].strftime("%Y%m%d")
        end = readings[-1]["tstamp"].strftime("%Y%m%d")
        output_path, ext = os.path.splitext(args.output_file)
        dire = os.path.dirname(output_path)
        base = os.path.basename(output_path)
        base = f"{base}_{start}_{end}{ext}"
        output_path = os.path.join(dire, base)
        with open(output_path, "w", newline="") as csvfile:
            fieldnames = readings[0].keys()
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames, delimiter=",")
            writer.writeheader()
            for item in readings:
                writer.writerow(item)
        log.info("readings written to %s", output_path)
    else:
        log.info("No readings found in the [%s - %s] period", args.start, args.end)



def cli_names(args: Namespace) -> None:
    summary_info, registry_info = collect_references(args.log_dir, args.start, args.end)
    with open(args.output_file, "w", newline="") as csvfile:
        writer = csv.writer(csvfile, delimiter="\t")
        writer.writerow(("name",  "#readings", "month", "from", "to", "registered?", "details"))
        for item in summary_info:
            writer.writerow(item)
    with open(args.registry_file, "w", newline="") as csvfile:
        writer = csv.writer(csvfile, delimiter=",")
        writer.writerow(("tstamp", "name", "mac", "model", "firmware", "zp1", "zp2", "zp3", "zp4", "place", "obs_type", "obs_name"))
        for item in registry_info:
            writer.writerow(item)


def add_args(parser: ArgumentParser) -> None:
    subparser = parser.add_subparsers(dest="command", required=True)
    p = subparser.add_parser(
        "readings",
        parents=[prs.name(), prs.logdir(), prs.ofile(), prs.batch(), prs.range()],
        help="Recover single photometer readings",
    )
    p.set_defaults(func=cli_readings)
    p = subparser.add_parser(
        "names",
        parents=[
            prs.logdir(),
            prs.ofile(),
            prs.rfile(),
            prs.range(),
        ],
        help="List photometers with recoverable readings",
    )
    p.set_defaults(func=cli_names)


def cli_main(args: Namespace) -> None:
    sqa_logging(args)
    args.func(args)
    engine_zpt.dispose()
    engine_tdb.dispose()


def main():
    execute(
        main_func=cli_main,
        add_args_func=add_args,
        name=__name__,
        version=__version__,
        description="TESSDB Log file readings recovery tool",
    )



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

from argparse import ArgumentParser

# ---------------------------
# Third-party library imports
# ----------------------------

from lica.validators import vdir


def name() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-n",
        "--name",
        type=str,
        required=True,
        help="photometer name",
    )
    return parser


def logdir() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ld",
        "--log-dir",
        type=vdir,
        required=True,
        help="Log files directory",
    )
    return parser


def ofile() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-of",
        "--output-file",
        type=str,
        required=True,
        help="Output CSV file",
    )
    return parser

def rfile() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-rf",
        "--registry-file",
        type=str,
        required=True,
        help="Output file with registry data",
    )
    return parser

def batch() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-bs",
        "--batch-size",
        type=int,
        default=30,
        help="Batch size when analyzing log files",
    )
    return parser

def range() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-st",
        "--start",
        type=int,
        default=None,
        metavar="<YYYYMMDD>",
        help="Earliest log file date (defaults to earliest available)",
    )
    parser.add_argument(
        "-en",
        "--end",
        type=int,
        default=None,
        metavar="<YYYYMMDD>",
        help="Latest log file date (defaults to latest available)",
    )
    return parser



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

import csv
import logging
import functools
from collections import defaultdict
from argparse import ArgumentParser, Namespace

# -------------------
# Third party imports
# -------------------

import decouple

from lica.cli import execute
from lica.tabulate import paging
from lica.jinja2 import render_from
from lica.sqlalchemy import sqa_logging


from tessdbapi.model import LocationInfo 
from tessdbapi.noasync.location import location_create, location_update, location_nearby, location_list
from tessdbapi.location_common import geolocate, geolocate_raw

# --------------
# local imports
# -------------

from .._version import __version__
from . import parser as prs
from .dao import engine, Session
from ..admin import TessDbServer

# ---------
# Constants
# ---------

CREATE_TEMPLATE = "location_create.sh.j2"

# ----------------
# Global variables
# ----------------

log = logging.getLogger(__name__.split(".")[-1])
package = ".".join(__name__.split(".")[:-2])

render = functools.partial(render_from, package)

host = decouple.config("TESSDB_ADMIN_HOST", default="localhost")
port = decouple.config("TESSDB_ADMIN_PORT", cast=int, default=8080)

# ================
# MAIN ENTRY POINT
# ================

def cli_reverse_lookup(args: Namespace) -> None:
    if args.raw:
        result = geolocate_raw(args.longitude, args.latitude)
    else:
        result = geolocate(args.longitude, args.latitude)
    result = [tup for tup in result.items()]
    paging(result, headers=("Name", "Value"), table_fmt="grid")


def cli_location_generate(args: Namespace) -> None:
    with open(args.input_csv, newline="") as csvfile:
        reader = csv.DictReader(csvfile)
        locations = [dict(row) for row in reader]
    log.info("found %d records", len(locations))
    loc_by_place = defaultdict(list)
    for obs in locations:
        loc_by_place[(obs["longitude"],obs["latitude"])].append(obs)
    for key, val in loc_by_place.items():
        if len(val) > 1 and key != ("",""):
            log.warning("Duplicate coordinate entries: %s", val)
        elif len(val) > 1 and key == ("",""):
            log.warning("Missing coordinate for %d entries", len(val))
    locations = [val[0] for key, val in loc_by_place.items() if key != ("","")]
    log.info("reduced to %d locations", len(locations))
    context = {"locations": locations}
    script = render(CREATE_TEMPLATE, context)
    with open(args.output_script, "w") as outfile:
        outfile.write(script)
    log.info("script '%s' generated", args.output_script)



def cli_location_create(args: Namespace) -> None:
    candidate = LocationInfo(
        longitude=args.longitude,
        latitude=args.latitude,
        height=args.height,
        place=args.place.strip() if args.place else None,
        town=args.town.strip() if args.town else None,
        sub_region=args.sub_region.strip() if args.sub_region else None,
        region=args.region.strip() if args.region else None,
        country=args.country.strip() if args.country else None,
        timezone=args.timezone.strip() if args.timezone else None,
    )
    with TessDbServer(host=host, port=port, test=args.test):
        with Session() as session:
            with session.begin():
                log.info("Registering location: %s", dict(candidate))
                if args.nearby is not None:
                    nearby_locs = location_nearby(session, candidate, limit=args.nearby)
                    if nearby_locs:
                        log.warning("Stopping here. New location is nearby these locations:")
                        for loc in nearby_locs:
                            log.warning(loc)
                        return
                location = location_create(
                    session,
                    candidate,
                    args.dry_run,
                )
            if location is not None:
                session.refresh(location)
                log.info(location)

def cli_location_update(args: Namespace) -> None:
    candidate = LocationInfo(
        longitude=args.longitude,
        latitude=args.latitude,
        height=args.height,
        place=args.place,
        town=args.town,
        sub_region=args.sub_region,
        region=args.region,
        country=args.country,
        timezone=args.timezone,
    )
    with TessDbServer(host=host, port=port, test=args.test):
        with Session() as session:
            with session.begin():
                log.info("Updating location: %s", dict(candidate))
                location_update(
                    session,
                    candidate,
                    args.dry_run,
                )

def cli_location_list(args: Namespace) -> None:
    with TessDbServer(host=host, port=port, test=args.test):
        with Session() as session:
            with session.begin():
                locations = location_list(session)
                locations = [ dict(location.__dict__) for location in locations]
                for location in locations:
                    del location["_sa_instance_state"]
                headers = locations[0].keys()
                result = [tup for tup in locations.items()]
                paging(result, headers=headers, table_fmt="grid")


def add_args(parser: ArgumentParser) -> None:
    subparser = parser.add_subparsers(dest="command", required=True)
    p = subparser.add_parser(
        "create",
        parents=[prs.coords(), prs.place(), prs.nominatim(), prs.nearby(), prs.dry(), prs.test()],
        help="Create new location",
    )
    p.set_defaults(func=cli_location_create)
    p = subparser.add_parser(
        "update",
        parents=[prs.coords(), prs.place(), prs.nominatim(), prs.dry(), prs.test()],
        help="Update existing location",
    )
    p.set_defaults(func=cli_location_update)
    p = subparser.add_parser(
        "reverse",
        parents=[prs.coords(),  prs.raw()],
        help="Location Reverse lookup operation",
    )
    p.set_defaults(func=cli_reverse_lookup)
    p = subparser.add_parser(
        "generate",
        parents=[
            prs.ifile(),
            prs.ofile(),
            prs.dry(),
        ],
        help="Generate location creation script from observers CSV",
    )
    p.set_defaults(func=cli_location_generate)
    p = subparser.add_parser(
        "list",
        parents=[prs.test()],
        help="List locations in database",
    )
    p.set_defaults(func=cli_location_list)


def cli_main(args: Namespace) -> None:
    sqa_logging(args)
    args.func(args)
    engine.dispose()


def main():
    execute(
        main_func=cli_main,
        add_args_func=add_args,
        name=__name__,
        version=__version__,
        description="TESSDB Location tool",
    )



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

import json
import asyncio
from datetime import datetime
import logging
import statistics
from itertools import pairwise
from typing import Any, Optional, Dict, AsyncIterator, TypeVar, List, Tuple

# ---------------------------
# Third-party library imports
# ----------------------------

import decouple
import aiomqtt
from aiomqtt.client import ProtocolVersion

# --------------
# local imports
# -------------


# ---------
# CONSTANTS
# ---------


# ------------------
# Additional Classes
# ------------------

# ----------------
# Global variables
# ----------------

log = logging.getLogger(__name__.split(".")[-2])
proto_log = logging.getLogger("MQTT")

T = TypeVar("T")

# -----------------
# Auxiliar functions
# ------------------


async def aioenumerate(aiter: AsyncIterator[T], start: int = 0) -> AsyncIterator[Tuple[int, T]]:
    idx = start
    async for item in aiter:
        yield idx, item
        idx += 1


# --------------
# The MQTT task
# --------------


def average_stdev(samples: List[Dict[str, Any]]) -> Tuple[float, float]:
    deltas = [
        (s2["T"] - s1["T"]).total_seconds() / (s2["seq"] - s1["seq"])
        for (s1, s2) in pairwise(samples)
    ]
    return statistics.mean(deltas), statistics.stdev(deltas)


async def subscriber(phot_name: str, limit: Optional[int]) -> None:
    interval = 5
    log.info("Starting MQTT subscriber")
    client = aiomqtt.Client(
        decouple.config("MQTT_HOST"),
        decouple.config("MQTT_PORT", cast=int),
        username=decouple.config("MQTT_USERNAME"),
        password=decouple.config("MQTT_PASSWORD"),
        identifier=decouple.config("MQTT_CLIENT_ID"),
        transport=decouple.config("MQTT_TRANSPORT"),
        logger=proto_log,
        keepalive=60,
        protocol=ProtocolVersion.V311,
    )
    topic = f"STARS4ALL/{phot_name}/reading"
    try:
        async with client:
            log.info("Subscribing to %s", topic)
            await client.subscribe(topic, qos=2)
            samples = list()
            async for i, message in aioenumerate(client.messages, start=1):
                tstamp = datetime.now()
                row = json.loads(message.payload.decode("utf-8"))
                if message.retain:
                    log.info("[%s] Discarding retained sample #%d", phot_name, row["seq"])
                    continue
                row["T"] = tstamp
                samples.append(row)
                log.info("[%s] Got sample #%d", phot_name, row["seq"])
                if len(samples) > 2:
                    mean, stdev = average_stdev(samples)
                    log.info(
                        "[%s] Tx: \u03bc = %.02f sec., \u03c3 = %.03f sec. over %d samples",
                        phot_name,
                        mean,
                        stdev,
                        len(samples),
                    )
                if limit is not None and i == limit:
                    break
    except json.JSONDecodeError:
        log.error("Invalid JSON in payload=%s", payload)
    except aiomqtt.MqttError:
        log.info(f"Connection lost; Reconnecting in {interval} seconds ...")
        await asyncio.sleep(interval)



from lica.sqlalchemy.noasync.dbase import create_engine_sessionclass

engine_tdb, TdbSession = create_engine_sessionclass(env_var="TESSDB_URL", tag="tessdb")
engine_zpt, ZptSession = create_engine_sessionclass(env_var="ZPTESS_URL", tag="zptess")

__all__ = ["engine_tdb", "TdbSession", "engine_zpt", "ZptSession"]



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

import csv
import logging
import functools
from itertools import batched, islice
from argparse import ArgumentParser, Namespace
from typing import Iterable, Dict, Any, List

# -------------------
# Third party imports
# -------------------

import decouple

from pydantic import ValidationError

from lica.cli import execute
from lica.jinja2 import render_from
from lica.sqlalchemy import sqa_logging

from tessdbdao import RegisterState, ReadingSource, TimestampSource
from tessdbapi.model import PhotometerInfo, ReadingInfo1c, ReadingInfo4c, ReadingInfo
from tessdbapi.noasync.photometer.register import (
    photometer_register,
    photometer_assign,
    photometer_fix_valid_since,
)
from tessdbapi.noasync.photometer.reading import photometer_batch_write

# --------------
# local imports
# -------------

from .._version import __version__
from . import parser as prs
from .dao import engine, Session
from ..admin import TessDbServer

# ----------------
# Global variables
# ----------------

package = ".".join(__name__.split(".")[:-2])
log = logging.getLogger(__name__.split(".")[-1])


host = decouple.config("TESSDB_ADMIN_HOST", default="localhost")
port = decouple.config("TESSDB_ADMIN_PORT", cast=int, default=8080)


CREATE_TEMPLATE = "photometer_register.sh.j2"

# ----------------
# Global variables
# ----------------


render = functools.partial(render_from, package)


def build_pydantic_models(batch: Iterable[Dict[str, Any]], every: int) -> List[ReadingInfo]:
    readings = list()
    for row in islice(batch, 0, None, every):
        if "freq4" not in row:
            readings.append(
                ReadingInfo1c(
                    tstamp=row["tstamp"],
                    tstamp_src=row.get("tstamp_src", TimestampSource.SUBSCRIBER),
                    name=row["name"],
                    sequence_number=row["sequence_number"],
                    box_temperature=row["box_temperature"],
                    sky_temperature=row["sky_temperature"],
                    signal_strength=row["signal_strength"],
                    hash=row.get("hash"),
                    freq1=row["freq1"],
                    mag1=row["mag1"],
                )
            )
        else:
            readings.append(
                ReadingInfo4c(
                    tstamp=row["tstamp"],
                    tstamp_src=row.get("tstamp_src", TimestampSource.SUBSCRIBER),
                    name=row["name"],
                    sequence_number=row["sequence_number"],
                    box_temperature=row["box_temperature"],
                    sky_temperature=row["sky_temperature"],
                    signal_strength=row["signal_strength"],
                    hash=row.get("hash"),
                    freq1=row["freq1"],
                    mag1=row["mag1"],
                    freq2=row["freq2"],
                    mag2=row["mag2"],
                    freq3=row["freq3"],
                    mag3=row["mag3"],
                    freq4=row["freq4"],
                    mag4=row["mag4"],
                )
            )
    return readings


# ================
# MAIN ENTRY POINT
# ================


def cli_photom_generate(args: Namespace) -> None:
    with open(args.input_csv, newline="") as csvfile:
        reader = csv.DictReader(csvfile, delimiter=",")
        photometers = [dict(row) for row in reader]
        log.info("Number of candidate photometers to register: %d", len(photometers))
        photometers = list(
            filter(
                lambda x: all(
                    [
                        x["firmware"],
                    ]
                ),
                photometers,
            )
        )
        log.info(
            "Number of candidate photometers to register after filtering: %d", len(photometers)
        )
        context = {"photometers": photometers}
        script = render(CREATE_TEMPLATE, context)
        with open(args.output_script, "w") as outfile:
            outfile.write(script)
        log.info("script '%s' generated", args.output_script)


def cli_photom_register(args: Namespace) -> None:
    try:
        candidate = PhotometerInfo(
            name=args.name,
            mac_address=args.mac_address,
            model=args.model,
            firmware=args.firmware,
            authorised=args.authorise,
            registered=RegisterState.MANUAL,
            zp1=args.zp1,
            filter1=args.filter1,
            offset1=args.offset1,
            zp2=args.zp2,
            filter2=args.filter2,
            offset2=args.offset2,
            zp3=args.zp3,
            filter3=args.filter3,
            offset3=args.offset3,
            zp4=args.zp4,
            filter4=args.filter4,
            offset4=args.offset4,
            tstamp=args.timestamp,
        )
    except ValidationError as e:
        log.error("Validation Error")
        log.info(e)
    else:
        with TessDbServer(host=host, port=port, test=args.test):
            with Session() as session:
                with session.begin():
                    log.info("Registering photometer: %s", dict(candidate))
                    photometer_register(
                        session,
                        candidate=candidate,
                        place=args.place,
                        observer_name=args.observer,
                        observer_type=args.type,
                        dry_run=args.dry_run,
                    )


def cli_photom_upload(args: Namespace) -> None:
    name = args.input_csv.split("_")[0]
    with open(args.input_csv) as csvfile:
        reader = csv.DictReader(csvfile)
        with TessDbServer(host=host, port=port, test=args.test):
            with Session() as session:
                for i, batch in enumerate(batched(reader, args.batch_size * args.every), start=1):
                    log.info(
                        "[%s] Writing batch #%d with %d samples, decimated to %d samples",
                        name,
                        i,
                        len(batch),
                        len(batch) // args.every,
                    )
                    subsampled = build_pydantic_models(batch, args.every)
                    photometer_batch_write(
                        session=session,
                        readings=subsampled,
                        auth_filter=False,
                        latest=False,
                        source=ReadingSource.IMPORTED,
                        dry_run=args.dry_run,
                    )


def cli_photom_assign(args: Namespace) -> None:
    with TessDbServer(host=host, port=port, test=args.test):
        with Session() as session:
            with session.begin():
                photometer_assign(
                    session=session,
                    phot_name=args.name,
                    place=args.place,
                    observer_name=args.observer,
                    observer_type=args.type,
                    update_readings=args.readings,
                    update_readings_since=args.since,
                    update_readings_until=args.until,
                    dry_run=args.dry_run,
                )


def cli_photom_valid_since(args: Namespace) -> None:
    with TessDbServer(host=host, port=port, test=args.test):
        with Session() as session:
            with session.begin():
                photometer_fix_valid_since(
                    session=session,
                    name=args.name,
                    valid_since=args.timestamp,
                    dry_run=args.dry_run,
                )


def add_args(parser: ArgumentParser) -> None:
    subparser = parser.add_subparsers(dest="command", required=True)
    p = subparser.add_parser(
        "register",
        parents=[prs.tstamp(), prs.photom(), prs.location(), prs.observer(), prs.dry(), prs.test()],
        help="Register a new photometer",
    )
    p.set_defaults(func=cli_photom_register)
    p = subparser.add_parser(
        "valid_since",
        parents=[prs.name(), prs.tstamp(), prs.dry(), prs.test()],
        help="fix valid_since in an already registered photometer",
    )
    p.set_defaults(func=cli_photom_valid_since)
    p = subparser.add_parser(
        "assign",
        parents=[prs.name(), prs.location(), prs.observer(), prs.readings(), prs.dry(), prs.test()],
        help="Assign a location and observer to a given photometer",
    )
    p.set_defaults(func=cli_photom_assign)
    p = subparser.add_parser(
        "generate",
        parents=[prs.ifile(), prs.ofile(), prs.dry()],
        help=("Generate photometers register script from photometers CSV",),
    )
    p.set_defaults(func=cli_photom_generate)
    p = subparser.add_parser(
        "upload",
        parents=[prs.ifile(), prs.batch(), prs.every(), prs.dry(), prs.test()],
        help=("Upload readings to database"),
    )
    p.set_defaults(func=cli_photom_upload)


def cli_main(args: Namespace) -> None:
    sqa_logging(args)
    args.func(args)
    engine.dispose()


def main():
    execute(
        main_func=cli_main,
        add_args_func=add_args,
        name=__name__,
        version=__version__,
        description="TESSDB Observer tool",
    )



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

from argparse import ArgumentParser

# ---------------------------
# Third-party library imports
# ----------------------------

from lica.validators import vfile, vdate
from tessdbdao import ObserverType, PhotometerModel


def test() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-te",
        "--test",
        default=False,
        action="store_true",
        help="Test Mode, do not pause tessdb-server",
    )
    return parser


def readings() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-re",
        "--readings",
        default=False,
        action="store_true",
        help="Update location and observer in readings too",
    )
    parser.add_argument(
        "-si",
        "--since",
        type=vdate,
        default=None,
        metavar="<YYYY-MM-DD>",
        help="update readings since (defaults to %(default)s)",
    )
    parser.add_argument(
        "-un",
        "--until",
        type=vdate,
        default=None,
        metavar="<YYYY-MM-DD>",
        help="update readings until (defaults to %(default)s)",
    )
    return parser


def name() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-na",
        "--name",
        type=str,
        required=True,
        help="Photometer name",
    )
    return parser


def every() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ev",
        "--every",
        type=int,
        default=1,
        help="Chooses 1 every N reading to insert into the database",
    )
    return parser


def batch() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-bs",
        "--batch-size",
        type=int,
        default=1440,  # One day of readings @ 1 reading/min
        help="Batch size when uploading to database",
    )
    return parser


def ifile() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ic",
        "--input-csv",
        type=vfile,
        required=True,
        help="Input CSV file",
    )
    return parser


def ofile() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-os",
        "--output-script",
        type=str,
        required=True,
        help="Output shell script",
    )
    return parser


def dry() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-dr",
        "--dry-run",
        default=False,
        action="store_true",
        help="Dry run, do not update database",
    )
    return parser


def observer() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ty",
        "--type",
        type=ObserverType,
        default=None,
        help="Observer Type (person, organization)",
    )
    parser.add_argument(
        "-ob",
        "--observer",
        type=str,
        default=None,
        help="Observer name",
    )
    return parser


def location() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-pl",
        "--place",
        type=str,
        default=None,
        help="Place name",
    )
    return parser


def tstamp() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ts",
        "--timestamp",
        type=vdate,
        default=None,
        metavar="<YYYY-MM-DDTHH:MM:SS>",
        help="Registration UTC timestamp (defaults to %(default)s = current UTC time)",
    )
    return parser


def photom() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ma",
        "--mac-address",
        type=str,
        required=True,
        help="Photometer MAC address",
    )
    parser.add_argument(
        "-na",
        "--name",
        type=str,
        required=True,
        help="Photometer name",
    )
    parser.add_argument(
        "-mo",
        "--model",
        type=PhotometerModel,
        default=PhotometerModel.TESSW,
        help="Photometer model",
    )
    parser.add_argument(
        "-fi",
        "--firmware",
        type=str,
        required=True,
        help="Photometer name",
    )
    parser.add_argument(
        "-au",
        "--authorise",
        default=False,
        action="store_true",
        help="Authorise to store readings",
    )
    parser.add_argument(
        "--zp1",
        type=float,
        required=True,
        help="Zero Point 1 [mag/arcsec^2]",
    )
    parser.add_argument(
        "-o1",
        "--offset1",
        type=float,
        default=0.0,
        help="Frequency offset 1 [Hz] (defaults to %(default)s)",
    )
    parser.add_argument(
        "-f1",
        "--filter1",
        type=str,
        default="UV/IR-740",
        help="Filter 1 (defaults to %(default)s)",
    )

    parser.add_argument(
        "--zp2",
        type=float,
        default=None,
        help="Zero Point 2 [mag/arcsec^2] (defaults to %(default)s)",
    )
    parser.add_argument(
        "-o2",
        "--offset2",
        type=float,
        default=None,
        help="Frequency offset 2 [Hz] (defaults to %(default)s)",
    )
    parser.add_argument(
        "-f2",
        "--filter2",
        type=str,
        default=None,
        help="Filter 2 (defaults to %(default)s)",
    )

    parser.add_argument(
        "--zp3",
        type=float,
        default=None,
        help="Zero Point 3 [mag/arcsec^2] (defaults to %(default)s)",
    )
    parser.add_argument(
        "-o3",
        "--offset3",
        type=float,
        default=None,
        help="Frequency offset 3 [Hz] (defaults to %(default)s)",
    )
    parser.add_argument(
        "-f3",
        "--filter3",
        type=str,
        default=None,
        help="Filter 3 (defaults to %(default)s)",
    )

    parser.add_argument(
        "--zp4",
        type=float,
        default=None,
        help="Zero Point 4 [mag/arcsec^2] (defaults to %(default)s)",
    )
    parser.add_argument(
        "-o4",
        "--offset4",
        type=float,
        default=None,
        help="Frequency offset 4 [Hz] (defaults to %(default)s)",
    )
    parser.add_argument(
        "-f4",
        "--filter4",
        type=str,
        default=None,
        help="Filter 4 (defaults to %(default)s)",
    )

    return parser



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

import csv
import logging
import functools
from argparse import ArgumentParser, Namespace

# -------------------
# Third party imports
# -------------------

from lica.asyncio.cli import execute

# --------------
# local imports
# -------------

from .._version import __version__
from . import parser as prs
from .mqtt import subscriber

# ---------
# Constants
# ---------


# ----------------
# Global variables
# ----------------

log = logging.getLogger(__name__.split(".")[-1])

# ================
# MAIN ENTRY POINT
# ================


async def cli_period_estimate(args: Namespace) -> None:
    await subscriber(args.name, args.num_samples)


def add_args(parser: ArgumentParser) -> None:
    subparser = parser.add_subparsers(dest="command", required=True)
    p = subparser.add_parser(
        "estimate",
        parents=[prs.phot()],
        help="Estimate photomter Tx period",
    )
    p.set_defaults(func=cli_period_estimate)


async def cli_main(args: Namespace) -> None:
    await args.func(args)


def main():
    execute(
        main_func=cli_main,
        add_args_func=add_args,
        name=__name__,
        version=__version__,
        description="TESSDB Tx period estimation tool",
    )



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

from argparse import ArgumentParser

# ---------------------------
# Third-party library imports
# ----------------------------

from lica.validators import vfile

def test() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-te",
        "--test",
        default=False,
        action="store_true",
        help="Test Mode, do not pause tessdb-server",
    )
    return parser

def dry() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-dr",
        "--dry-run",
        default=False,
        action="store_true",
        help="Dry run, do not update database",
    )
    return parser


def place() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-pl",
        "--place",
        type=str,
        required=True,
        help="Place name",
    )
    return parser


def nominatim() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-to",
        "--town",
        type=str,
        default=None,
        help="Population Center",
    )
    parser.add_argument(
        "-su",
        "--sub-region",
        type=str,
        default=None,
        help="Province, county, etcr",
    )
    parser.add_argument(
        "-re",
        "--region",
        type=str,
        default=None,
        help="Federal state, comunidad autonoma",
    )
    parser.add_argument(
        "-co",
        "--country",
        type=str,
        default=None,
        help="Country",
    )
    parser.add_argument(
        "-tz",
        "--timezone",
        type=str,
        default=None,
        help="Timezone",
    )
    return parser


def coords() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-lo",
        "--longitude",
        type=float,
        required=True,
        help="Longitude (decimal degrees)",
    )
    parser.add_argument(
        "-la",
        "--latitude",
        type=float,
        required=True,
        help="Latitude (decimal degrees)",
    )
    parser.add_argument(
        "-he",
        "--height",
        type=float,
        default=None,
        help="Meters above sea level",
    )
    return parser

def raw() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ra",
        "--raw",
        default=False,
        action="store_true",
        help="Raw Nominatim metadata",
    )
    return parser


def ifile() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ic",
        "--input-csv",
        type=vfile,
        required=True,
        help="Input CSV file",
    )
    return parser

def ofile() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-os",
        "--output-script",
        type=str,
        required=True,
        help="Output shell script",
    )
    return parser

def nearby() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ne",
        "--nearby",
        type=float,
        default=None,
        help="Nearby distance to check first (defaults to %(default)s )",
    )
    return parser


# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

import os
import re
import glob
import logging
import datetime  # necessary for eval
from itertools import batched
from collections import defaultdict
import functools

from typing import Optional, List, Dict, Mapping, Tuple, Any

# -------------------
# Third party imports
# -------------------

from sqlalchemy import select, func

from tessdbdao import TimestampSource
from tessdbdao.noasync import NameMapping
from zptessdao.noasync import SummaryView


# --------------
# local imports
# -------------

from .dao import TdbSession, ZptSession

# ----------------
# Global variables
# ----------------

package = ".".join(__name__.split(".")[:-1])
log = logging.getLogger(__name__.split(".")[-1])

TSTAMP_PATTERN = re.compile("(.+)'tstamp_src': <TimestampSource.+: '(.+)'>(.+)")


def is_tess4c(readings: Mapping[str, Any]) -> bool:
    return "freq4" in readings


def windowed3(start: Optional[int], end: Optional[int], path: str) -> bool:
    path = os.path.basename(path)
    if start is not None and path < f"tessdb.log-{start}":
        return False
    if end is not None and path > f"tessdb.log-{end}":
        return False
    return True


def filtered_log_files(log_dir: str, start: Optional[int], end: Optional[int]) -> List[str]:
    template = os.path.join(log_dir, "tessdb.log-????????")
    files = glob.iglob(template)
    windowed = functools.partial(windowed3, start, end)
    return sorted(filter(windowed, files), reverse=True)


def context(registration_needed: bool, readings: List[Dict[str, Any]]) -> Dict[str, Any]:
    ctx = dict()
    ctx["registration_needed"] = registration_needed
    ctx["model"] = "TESS4C" if is_tess4c(readings[0]) else "TESS-W"
    ctx["readings"] = list(enumerate(batched(readings, 10000), start=1))
    return ctx


def fix_tstamp_src_enum(value: str) -> str:
    matchobj = TSTAMP_PATTERN.search(value)
    groups = matchobj.groups()
    value = groups[0] + f"'tstamp_src': '{groups[1]}'" + groups[2]
    return value


# The new format in tessdb-server-ng
def fix_old_format(reading: dict[str, Any]) -> dict[str, Any]:
    del reading["rev"]
    reading["sequence_number"] = reading["seq"]
    del reading["seq"]
    reading["signal_strength"] = reading["wdBm"]
    del reading["wdBm"]
    reading["sky_temperature"] = reading["tsky"]
    del reading["tsky"]
    reading["box_temperature"] = reading["tamb"]
    del reading["tamb"]
    reading["longitude"] = None
    reading["latitude"] = None
    reading["elevation"] = None
    reading["altitude"] = None
    reading["azimuth"] = None
    return reading


def collect_readings_for(
    name: str, log_dir: str, start: int, end: int, batch_size: int
) -> List[Dict[str, Any]]:
    # This pattern captures both old and new log format when tessdb-server-ng was placed in production
    pattern = rf".{{42,60}} No TESS {name} registered ! => (.+)"
    pattern = re.compile(pattern)
    files = filtered_log_files(log_dir, start, end)
    readings = list()
    for i, batch in enumerate(batched(files, batch_size), start=1):
        batched_readings = list()
        batched_lengths = list()
        N = len(batch)
        for j, path in enumerate(batch, start=1):
            with open(path, "r") as fd:
                file_readings = list()
                for line in fd.readlines():
                    matchobj = pattern.search(line)
                    if matchobj:
                        found = matchobj.groups()[0]
                        try:
                            reading = eval(found)
                        except SyntaxError:
                            found = fix_tstamp_src_enum(found)
                            reading = eval(found)
                        if "valid_current" in reading:
                            del reading["valid_current"]
                        if "seq" in reading:
                            reading = fix_old_format(reading)
                        reading["tstamp"] = reading["tstamp"].replace(microsecond=0)
                        file_readings.append(reading)
                batched_readings.extend(file_readings)
                batched_lengths.append(len(batched_readings))
                log.info(
                    "[%s] Analyzing batch #%d (%02d/%02d), %s for readings, %d found",
                    name,
                    i,
                    j,
                    N,
                    path,
                    len(file_readings),
                )
        readings.extend(batched_readings)
        if all([batched_lengths[0] == n for n in batched_lengths]):
            log.warn("Stopping analysis in batch #%d . No activity was found.", i)
            break
    log.info("Sorting %s readings by timestamp", name)
    return sorted(readings, key=lambda x: x["tstamp"])


def already_registered(name: str) -> bool:
    with TdbSession() as session:
        query = select(func.count("*")).select_from(NameMapping).where(NameMapping.name == name)
        N = session.scalars(query).one()
        log.info("Found %d entries for %s", N, name)
        if N > 0:
            query = select(NameMapping).where(NameMapping.name == name)
            result_set = session.scalars(query)
            for result in result_set:
                log.info(result)
    return N > 0


def find_photometer_details(name: str) -> Tuple[str, float]:
    with ZptSession() as session:
        query = select(SummaryView.mac, SummaryView.zero_point, SummaryView.firmware).where(
            SummaryView.name == name
        )
        result_set = session.execute(query)
    return result_set


def to_date(value: str) -> str:
    return datetime.datetime.strptime(value, "%Y%m%d").strftime("%Y-%m-%dT%H:%M:%S")


def collect_references(
    log_dir: str, start: int, end: int
) -> List[Tuple[str, str, int, str, str, bool, Any]]:
    pattern = r".{42,60} No TESS (stars\d{1,8}) registered ! => "
    pattern = re.compile(pattern)
    names = set()
    counts = dict()
    files_per_name = defaultdict(set)
    files = filtered_log_files(log_dir, start, end)
    for path in files:
        with open(path, "r") as fd:
            log.info("Analyzing %s, %d photometers found so far", path, len(names))
            for line in reversed(fd.readlines()):
                matchobj = pattern.search(line)
                if matchobj:
                    name = matchobj.groups()[0]
                    names.add(name)
                    counts[name] = counts.get(name, 0) + 1
                    files_per_name[name].add(os.path.basename(path))

    summary_info = [
        (
            name,
            f"{counts[name]:8d}",
            sorted(files_per_name[name])[0].split("-")[-1][:-2],
            sorted(files_per_name[name])[0],
            sorted(files_per_name[name])[-1],
            already_registered(name),
            tuple(find_photometer_details(name)),
        )
        for name in names
    ]
    summary_info = sorted(summary_info, reverse=True, key=lambda x: x[4])
    # log.info(summary_info)
    # "tstamp", "name", "mac", "model", "firmware", "zp1", "zp2", "zp3", "zp4", "place", "obs_type", "obs_name")
    registry_info = [
        (
            to_date(item[3][11:]),
            item[0],
            item[-1][0][0],
            "TESS-W",
            item[-1][0][2],
            item[-1][0][1],
            None,
            None,
            None,
            None,
            None,
        )
        for item in summary_info
        if len(item[-1]) == 1
    ]
    return summary_info, registry_info



import pytest
import logging
import requests
from datetime import datetime
from tess.admin import TessDbServer, TooCloseError

bad_t = datetime.now().replace(hour=11, minute=49, second=0)
good_t = datetime.now().replace(hour=12, minute=31, second=0)

log = logging.getLogger(__name__.split(".")[-1])


# Different window sizes
fail_conn_data = (("localhost",8081),("calyx.hst.ucm.es",8080))
ok_conn_data = (("localhost",8080),)


@pytest.fixture(params=fail_conn_data)
def fail_server(request) -> TessDbServer:
    return TessDbServer(host=request.param[0], port=request.param[1], timestamp=good_t)

@pytest.fixture(params=ok_conn_data)
def ok_server(request) -> TessDbServer:
    return TessDbServer(host=request.param[0], port=request.param[1], timestamp=good_t)

@pytest.fixture(params=fail_conn_data)
def noop_server(request) -> TessDbServer:
    return TessDbServer(host=request.param[0], port=request.param[1], test=True)

@pytest.fixture(params=ok_conn_data)
def fail_time(request) -> TessDbServer:
    return TessDbServer(host=request.param[0], port=request.param[1], timestamp=bad_t)


def test_enter_1(fail_server):
    with pytest.raises(Exception) as exc_info: 
        fail_server.__enter__()
    assert exc_info.type in (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout)
    log.info(exc_info.type)

def test_enter_2(fail_time):
    with pytest.raises(Exception) as exc_info: 
        fail_time.__enter__()
    assert exc_info.type is TooCloseError
    log.info(exc_info.type)

def test_enter_3(noop_server):
    noop_server.__enter__()
    

def test_exit_1(fail_server):
    with pytest.raises(Exception) as exc_info: 
        fail_server.__exit__(None, None, None)
    log.info(exc_info.type)
    assert exc_info.type in (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout)

def test_exit_3(noop_server):
    noop_server.__exit__(None, None, None)


def test_fail_with_1(fail_server):
    with pytest.raises(Exception) as exc_info:
        with fail_server:
            log.info("should never print this")
    log.info(exc_info.type)
    assert exc_info.type in (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout)

def test_fail_with_2(fail_time):
    with pytest.raises(Exception) as exc_info:
        with fail_time:
            log.info("should never print this")
    log.info(exc_info.type)
    assert exc_info.type is TooCloseError

def test_ok_with(ok_server):
    with ok_server:
        log.info("Doing DB stuff")

def test_noop_with(noop_server):
    with noop_server:
        log.info("Doing DB stuff")


# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

from argparse import ArgumentParser

# ---------------------------
# Third-party library imports
# ----------------------------


def phot() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-na",
        "--name",
        type=str,
        required=True,
        help="photometer name",
    )
    parser.add_argument(
        "-ns",
        "--num-samples",
        type=int,
        default=None,
        help="Number of samples (defaults to %(default)s = indefinite",
    )
    return parser



# ----------------------------------------------------------------------
# Copyright (c) 2024 Rafael Gonzalez.
#
# See the LICENSE file for details
# ----------------------------------------------------------------------

# --------------------
# System wide imports
# -------------------

from argparse import ArgumentParser

# ---------------------------
# Third-party library imports
# ----------------------------

from tessdbdao import ObserverType
from lica.validators import vfile

def test() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-te",
        "--test",
        default=False,
        action="store_true",
        help="Test Mode, do not pause tessdb-server",
    )
    return parser

def dry() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-dr",
        "--dry-run",
        default=False,
        action="store_true",
        help="Dry run, do not update database",
    )
    return parser

def fix() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-fx",
        "--fix",
        default=False,
        action="store_true",
        help="Fix latest entry",
    )
    return parser

def ifile() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ic",
        "--input-csv",
        type=vfile,
        required=True,
        help="Input CSV file",
    )
    return parser

def ofile() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-os",
        "--output-script",
        type=str,
        required=True,
        help="Output shell script",
    )
    return parser

def observer() -> ArgumentParser:
    parser = ArgumentParser(add_help=False)
    parser.add_argument(
        "-ty",
        "--type",
        type=ObserverType,
        default=None,
        help="Observer Type (person, organization)",
    )
    parser.add_argument(
        "-na",
        "--name",
        type=str,
        default=None,
        help="Observer name",
    )
    parser.add_argument(
        "-af",
        "--affiliation",
        type=str,
        default=None,
        help="Federal state, comunidad autonoma",
    )
    parser.add_argument(
        "-ac",
        "--acronym",
        type=str,
        default=None,
        help="Affiliation / Organization acronym",
    )
    parser.add_argument(
        "-we",
        "--website-url",
        type=str,
        default=None,
        help="Web site URL",
    )
    parser.add_argument(
        "-em",
        "--email",
        type=str,
        default=None,
        help="Contact email",
    )
    return parser
